package dev.shadowsoffire.apotheosis.adventure.compat;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import dev.shadowsoffire.apotheosis.Apoth;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import dev.shadowsoffire.apotheosis.adventure.AdventureModule;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import dev.shadowsoffire.apotheosis.adventure.affix.salvaging.SalvagingRecipe;
import dev.shadowsoffire.apotheosis.adventure.compat.ApothSmithingCategory;
import dev.shadowsoffire.apotheosis.adventure.compat.GemCuttingCategory;
import dev.shadowsoffire.apotheosis.adventure.compat.SalvagingCategory;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.AddSocketsRecipe;
import dev.shadowsoffire.apotheosis.adventure.socket.ReactiveSmithingRecipe;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.Gem;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemItem;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemRegistry;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.function.Function;
import mezz.jei.api.IModPlugin;
import mezz.jei.api.JeiPlugin;
import mezz.jei.api.constants.VanillaTypes;
import mezz.jei.api.ingredients.IIngredientType;
import mezz.jei.api.ingredients.subtypes.IIngredientSubtypeInterpreter;
import mezz.jei.api.recipe.RecipeType;
import mezz.jei.api.recipe.category.IRecipeCategory;
import mezz.jei.api.registration.IRecipeCatalystRegistration;
import mezz.jei.api.registration.IRecipeCategoryRegistration;
import mezz.jei.api.registration.IRecipeRegistration;
import mezz.jei.api.registration.ISubtypeRegistration;
import net.minecraft.client.Minecraft;
import net.minecraft.network.chat.Component;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.util.RandomSource;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.crafting.RecipeType;
import net.minecraft.world.item.crafting.SmithingRecipe;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.levelgen.LegacyRandomSource;

@JeiPlugin
public class AdventureJEIPlugin implements IModPlugin {
  public static final RecipeType<SmithingRecipe> APO_SMITHING = RecipeType.create("apotheosis", "smithing", AdventureModule.ApothSmithingRecipe.class);
  
  public static final RecipeType<SalvagingRecipe> SALVAGING = RecipeType.create("apotheosis", "salvaging", SalvagingRecipe.class);
  
  public static final RecipeType<GemCuttingCategory.GemCuttingRecipe> GEM_CUTTING = RecipeType.create("apotheosis", "gem_cutting", GemCuttingCategory.GemCuttingRecipe.class);
  
  public ResourceLocation getPluginUid() {
    return new ResourceLocation("apotheosis", "adventure_module");
  }
  
  public void registerRecipes(IRecipeRegistration reg) {
    if (!Apotheosis.enableAdventure)
      return; 
    ItemStack gem = new ItemStack((ItemLike)Adventure.Items.GEM.get());
    Gem gemObj = (Gem)GemRegistry.INSTANCE.getRandomItem((RandomSource)new LegacyRandomSource(1854L));
    GemItem.setGem(gem, gemObj);
    AffixHelper.setRarity(gem, gemObj.getMaxRarity());
    reg.addIngredientInfo(gem, (IIngredientType)VanillaTypes.ITEM_STACK, new Component[] { (Component)Component.m_237115_("info.apotheosis.socketing") });
    reg.addIngredientInfo(new ItemStack((ItemLike)Adventure.Items.GEM_DUST.get()), (IIngredientType)VanillaTypes.ITEM_STACK, new Component[] { (Component)Component.m_237115_("info.apotheosis.gem_crushing") });
    reg.addIngredientInfo(new ItemStack((ItemLike)Adventure.Items.SIGIL_OF_UNNAMING.get()), (IIngredientType)VanillaTypes.ITEM_STACK, new Component[] { (Component)Component.m_237115_("info.apotheosis.unnaming") });
    ApothSmithingCategory.registerExtension(AddSocketsRecipe.class, (ApothSmithingCategory.Extension)new AddSocketsExtension());
    Objects.requireNonNull(ReactiveSmithingRecipe.class);
    reg.addRecipes(APO_SMITHING, (Minecraft.m_91087_()).f_91073_.m_7465_().m_44013_(RecipeType.f_44113_).stream().filter(ReactiveSmithingRecipe.class::isInstance).toList());
    List<SalvagingRecipe> salvagingRecipes = new ArrayList<>((Minecraft.m_91087_()).f_91073_.m_7465_().m_44013_(Apoth.RecipeTypes.SALVAGING));
    salvagingRecipes.sort(Comparator.comparingInt(recipe -> recipe.getOutputs().stream().mapToInt(SalvagingRecipe.OutputData::getMax).max().orElse(0)));
    reg.addRecipes(SALVAGING, salvagingRecipes);
    List<GemCuttingCategory.GemCuttingRecipe> gemCutRecipes = new ArrayList<>();
    for (Gem g : GemRegistry.INSTANCE.getValues()) {
      LootRarity r = (LootRarity)RarityRegistry.getMinRarity().get();
      LootRarity max = (LootRarity)RarityRegistry.getMaxRarity().get();
      while (r != max) {
        if (g.clamp(r) == r)
          gemCutRecipes.add(new GemCuttingCategory.GemCuttingRecipe(g, r)); 
        r = r.next();
      } 
    } 
    reg.addRecipes(GEM_CUTTING, gemCutRecipes);
  }
  
  public void registerCategories(IRecipeCategoryRegistration reg) {
    if (!Apotheosis.enableAdventure)
      return; 
    reg.addRecipeCategories(new IRecipeCategory[] { (IRecipeCategory)new ApothSmithingCategory(reg.getJeiHelpers().getGuiHelper()) });
    reg.addRecipeCategories(new IRecipeCategory[] { (IRecipeCategory)new SalvagingCategory(reg.getJeiHelpers().getGuiHelper()) });
    reg.addRecipeCategories(new IRecipeCategory[] { (IRecipeCategory)new GemCuttingCategory(reg.getJeiHelpers().getGuiHelper()) });
  }
  
  public void registerRecipeCatalysts(IRecipeCatalystRegistration reg) {
    if (!Apotheosis.enableAdventure)
      return; 
    reg.addRecipeCatalyst(new ItemStack((ItemLike)Blocks.f_50625_), new RecipeType[] { APO_SMITHING });
    reg.addRecipeCatalyst(new ItemStack((ItemLike)Adventure.Blocks.SALVAGING_TABLE.get()), new RecipeType[] { SALVAGING });
    reg.addRecipeCatalyst(new ItemStack((ItemLike)Adventure.Blocks.GEM_CUTTING_TABLE.get()), new RecipeType[] { GEM_CUTTING });
  }
  
  public void registerItemSubtypes(ISubtypeRegistration reg) {
    if (!Apotheosis.enableAdventure)
      return; 
    reg.registerSubtypeInterpreter((Item)Adventure.Items.GEM.get(), (IIngredientSubtypeInterpreter)new GemSubtypes());
  }
  
  private static final List<ItemStack> DUMMY_INPUTS = Arrays.<Item>asList(new Item[] { Items.f_42430_, Items.f_42390_, Items.f_42428_, Items.f_42469_, Items.f_42713_ }).stream().map(ItemStack::new).toList();
}
